# coding=utf-8
# Copyright 2022 The OpenBMB Team The HuggingFace Inc. team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" PyTorch CpmBee model."""
import copy
import math
from collections import UserDict
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

import torch
import torch.nn as nn

from transformers.generation.beam_search import BeamHypotheses, BeamSearchScorer
from transformers.generation.streamers import BaseStreamer
from transformers.generation.utils import (
    GenerationConfig,
    LogitsProcessorList,
    StoppingCriteriaList,
    dist,
    inspect,
    is_deepspeed_zero3_enabled,
    warnings,
)
from transformers.modeling_outputs import BaseModelOutputWithPast, CausalLMOutputWithPast, ModelOutput
from transformers.modeling_utils import PreTrainedModel
from transformers.utils import add_code_sample_docstrings, add_start_docstrings, add_start_docstrings_to_model_forward, logging
from .configuration_cpmbee import CpmBeeConfig
from .tokenization_viscpmbee import VisCpmBeeTokenizer


logger = logging.get_logger(__name__)

_CHECKPOINT_FOR_DOC = "openbmb/cpm-bee-10b"
_CONFIG_FOR_DOC = "CpmBeeConfig"

CPMBEE_PRETRAINED_MODEL_ARCHIVE_LIST = [
    "openbmb/cpm-bee-10b",
    "openbmb/cpm-bee-5b",
    "openbmb/cpm-bee-2b",
    "openbmb/cpm-bee-1b",
    # See all CPMBee models at https://huggingface.co/models?filter=cpmbee
]


class CpmBeeLinear(nn.Linear):
    def __init__(self, dim_in, dim_out, dtype):
        """
        Construct a linear for CPMBee. It contains a scale operation.
        """
        super().__init__(dim_in, dim_out, bias=False)
        self.dim_in = self.in_features = dim_in
        self.dim_out = self.out_features = dim_out

        self.weight = torch.nn.parameter.Parameter(torch.empty((dim_out, dim_in), dtype=dtype))

    def forward(self, x: torch.Tensor):
        """
        Args:
            x (`torch.Tensor` of shape `(batch, seq_len, dim_in)`): The input of linear layer
        Returns:
            `torch.Tensor` of shape `(batch, seq_len, dim_out)`: The output of the linear transform y.
        """
        x = nn.functional.linear(x, self.weight)
        x = x / math.sqrt(self.dim_in)
        return x


class CpmBeeLayerNorm(nn.Module):
    """
    We use Root Mean Square (RMS) Layer Normalization, please see https://arxiv.org/abs/1910.07467 for details."
    """

    def __init__(self, config: CpmBeeConfig):
        super().__init__()

        self.eps = config.eps
        self.dim_norm = config.hidden_size
        self.weight = nn.Parameter(torch.empty(config.hidden_size, dtype=config.torch_dtype))

    def forward(self, hidden_states: torch.Tensor):
        """
        Args:
            hidden_states (`torch.Tensor` of shape `(batch, seq_len, dim_in)`)
        """
        if hidden_states.size(-1) != self.dim_norm:
            raise AssertionError("hidden_states.size(-1) != self.dim_norm")
        old_dtype = hidden_states.dtype
        variance = hidden_states.to(torch.float32).pow(2).mean(dim=-1, keepdim=True)
        hidden_states = (hidden_states * torch.rsqrt(variance + self.eps)).to(old_dtype) * self.weight
        return hidden_states


class CpmBeeAttention(nn.Module):
    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        self.dim_model = config.hidden_size
        self.num_heads = config.num_attention_heads
        self.dim_head = config.dim_head

        self.project_q = CpmBeeLinear(self.dim_model, self.num_heads * self.dim_head, dtype=config.torch_dtype)
        self.project_k = CpmBeeLinear(self.dim_model, self.num_heads * self.dim_head, dtype=config.torch_dtype)
        self.project_v = CpmBeeLinear(self.dim_model, self.num_heads * self.dim_head, dtype=config.torch_dtype)

        self.attention_out = CpmBeeLinear(self.num_heads * self.dim_head, self.dim_model, dtype=config.torch_dtype)

        self.softmax = torch.nn.Softmax(dim=-1)

        if config.dropout_p is not None:
            self.dropout = torch.nn.Dropout(p=config.dropout_p)
        else:
            self.dropout = None

    def forward(
        self,
        hidden_q: torch.Tensor,
        hidden_kv: torch.Tensor,
        attention_mask: torch.BoolTensor,
        position_bias: torch.Tensor,
        output_attentions: Optional[bool] = False,
        past_key_values: Optional[Tuple[torch.Tensor, torch.Tensor]] = None,
        use_cache: Optional[bool] = None,
    ):
        """
        Args:
            hidden_q (`torch.Tensor`):
                Input of transformer block(self-attention block). It can be the raw embedding of a batch of sequences.
            hidden_kv (`torch.Tensor` of shape `(batch, len_k, dim_model)`)):
                Tensor *key_value* and *query* of shape `(batch, len_k, dim_model)`
            attention_mask (`torch.Tensor` of shape `(batch, len_seq, len_seq)`):
                Avoid invalid areas to participate in the calculation of self-attention.
            position_bias (`torch.Tensor` of shape `(batch, len_seq, len_seq)`):
                Provide positional information to self-attention block.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers.
            past_key_values (`Tuple[torch.Tensor, torch.Tensor]`, *optional*):
                Cached past key and value projection states.
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding
                (see `past_key_values`).
        """
        batch_size = hidden_q.size(0)
        len_q = hidden_q.size(1)
        len_k = hidden_kv.size(1)

        query = self.project_q(hidden_q)
        key = self.project_k(hidden_kv)
        value = self.project_v(hidden_kv)

        query = query.view(batch_size, len_q, self.num_heads, self.dim_head).permute(0, 2, 1, 3)
        key = key.view(batch_size, len_k, self.num_heads, self.dim_head).permute(0, 2, 1, 3)
        value = value.view(batch_size, len_k, self.num_heads, self.dim_head).permute(0, 2, 1, 3)

        if past_key_values is not None:
            key = torch.cat([past_key_values[0], key], dim=-2)
            value = torch.cat([past_key_values[1], value], dim=-2)
            len_k = key.size(-2)

        # (batch_size, num_heads, len_q, dim_head) @ (batch_size, num_heads, dim_head, len_k) -> (batch_size, num_heads, len_q, len_k)
        score = torch.matmul(query, key.transpose(-1, -2)) / math.sqrt(self.dim_head)
        score = score + position_bias

        score = torch.masked_fill(
            score,
            attention_mask.view(batch_size, 1, len_q, len_k) == torch.tensor(False),
            torch.scalar_tensor(float("-inf"), device=score.device, dtype=score.dtype),
        )
        score = self.softmax(score)

        score = torch.masked_fill(
            score,
            attention_mask.view(batch_size, 1, len_q, len_k) == torch.tensor(False),
            torch.scalar_tensor(0, device=score.device, dtype=score.dtype),
        )
        if output_attentions:
            attn_weights = score
        else:
            attn_weights = None

        if self.dropout is not None:
            score = self.dropout(score)

        # (batch_size, num_heads, len_q, len_k) @ (batch_size, num_heads, len_k, dim_head) -> (batch_size, num_heads, len_q, dim_head)
        score = torch.matmul(score, value)

        score = score.view(batch_size, self.num_heads, len_q, self.dim_head).permute(0, 2, 1, 3)
        score = score.contiguous().view(batch_size, len_q, self.num_heads * self.dim_head)

        score = self.attention_out(score)

        past_key_values = None
        if use_cache:
            past_key_values = (key, value)

        return score, attn_weights, past_key_values


class CpmBeeSelfAttentionBlock(nn.Module):
    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        self.layernorm_before_attention = CpmBeeLayerNorm(config)
        self.self_attention = CpmBeeAttention(config)
        if config.dropout_p:
            self.dropout = torch.nn.Dropout(config.dropout_p)
        else:
            self.dropout = None

    def forward(
        self,
        hidden_states: torch.Tensor,
        attention_mask: torch.Tensor,
        position_bias: Optional[torch.Tensor] = None,
        output_attentions: Optional[bool] = False,
        past_key_values: Optional[Tuple[torch.Tensor, torch.Tensor]] = None,
        use_cache: Optional[bool] = None,
    ):
        """
        Args:
            hidden_states (`torch.Tensor` of shape `(batch, len_seq, dim_model)`):
                Input of transformer block(self-attention block). It can be the raw embedding of a batch of sequences.
            attention_mask (`torch.Tensor` of shape `(batch, len_seq, len_seq)`):
                Avoid invalid areas to participate in the calculation of self-attention.
            position_bias (`torch.Tensor` of shape `(batch, len_seq, len_seq)`):
                Provide positional information to self-attention block.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers.
            past_key_values (`Tuple(torch.FloatTensor)`, *optional*):
                Cached past key and value projection states.
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding
                (see `past_key_values`).
        """
        outputs = self.layernorm_before_attention(hidden_states)
        outputs = self.self_attention(
            outputs, outputs, attention_mask, position_bias, output_attentions, past_key_values, use_cache
        )

        outputs, attn_weights, current_key_value = outputs

        if self.dropout is not None:
            outputs = self.dropout(outputs)
        hidden_states = (hidden_states + outputs) / 1.05

        return hidden_states, attn_weights, current_key_value


class CpmBeeDenseGatedACT(nn.Module):
    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        self.w_0 = CpmBeeLinear(config.hidden_size, config.dim_ff, dtype=config.torch_dtype)
        self.w_1 = CpmBeeLinear(config.hidden_size, config.dim_ff, dtype=config.torch_dtype)
        self.act = torch.nn.GELU()

    def forward(self, hidden_states: torch.Tensor):
        """Transform an input tensor from one feature space to another via a nonlinear operation

        Args:
            hidden_states (`torch.Tensor` of shape `(batch, seq_len, dim_in)`)
        """
        gate_score = self.act(self.w_0(hidden_states))
        hidden_states = self.w_1(hidden_states)

        hidden_states = gate_score * hidden_states
        return hidden_states


class CpmBeeFeedForward(nn.Module):
    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        self.w_in = CpmBeeDenseGatedACT(config)
        if config.dropout_p is not None:
            self.dropout = torch.nn.Dropout(config.dropout_p)
        else:
            self.dropout = None

        self.w_out = CpmBeeLinear(config.dim_ff, config.hidden_size, dtype=config.torch_dtype)

    def forward(self, hidden_states: torch.Tensor):
        """
        Args:
            hidden_states (`torch.Tensor` of shape `(batch, seq_len, dim_in)`)
        """
        hidden_states = self.w_in(hidden_states)

        if self.dropout is not None:
            hidden_states = self.dropout(hidden_states)

        hidden_states = self.w_out(hidden_states)

        return hidden_states


class CpmBeeFFNBlock(nn.Module):
    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        self.layernorm_before_ffn = CpmBeeLayerNorm(config)
        self.ffn = CpmBeeFeedForward(config)
        if config.dropout_p:
            self.dropout = torch.nn.Dropout(config.dropout_p)
        else:
            self.dropout = None

    def forward(
        self,
        hidden_states: torch.Tensor,
    ):
        """
        Args:
            hidden_states (`torch.Tensor` of shape `(batch, len_seq, dim_model)`):
                Hidden states before feed forward layer.
        """
        ln_outputs = self.layernorm_before_ffn(hidden_states)
        outputs = self.ffn(ln_outputs)
        if self.dropout is not None:
            outputs = self.dropout(outputs)
        hidden_states = (hidden_states + outputs) / 1.05
        return hidden_states


class CpmBeeTransformerBlock(nn.Module):
    def __init__(self, config: CpmBeeConfig, mask_att: bool = False, mask_ffn: bool = False):
        super().__init__()
        self.mask_att = mask_att
        self.mask_ffn = mask_ffn

        if not self.mask_att:
            self.self_att = CpmBeeSelfAttentionBlock(config)
        if not self.mask_ffn:
            self.ffn = CpmBeeFFNBlock(config)

    def forward(
        self,
        hidden_states: torch.Tensor,
        attention_mask: torch.Tensor,
        position_bias: Optional[torch.Tensor] = None,
        output_attentions: Optional[bool] = False,
        past_key_values: Optional[Tuple[torch.Tensor, torch.Tensor]] = None,
        use_cache: Optional[bool] = None,
    ):
        """
        Args:
            hidden_states (`torch.Tensor`):
                Input to the layer of shape `(batch, seq_len, dim_model)`
            attention_mask (`torch.Tensor`):
                Avoid invalid areas to participate in the calculation of shape `(batch, seq_len, seq_len)`
            position_bias (`torch.Tensor`):
                Provides position information to attention mechanism of shape `(num_heads, seq_len, seq_len)`
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers.
            past_key_values (`Tuple[torch.Tensor, torch.Tensor])`, *optional*):
                Cached past key and value projection states
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding
                (see `past_key_values`).
        """
        if not self.mask_att:
            hidden_states = self.self_att(
                hidden_states,
                attention_mask=attention_mask,
                position_bias=position_bias,
                output_attentions=output_attentions,
                past_key_values=past_key_values,
                use_cache=use_cache,
            )

            hidden_states, attn_weights, current_key_value = hidden_states
        else:
            attn_weights, current_key_value = None, (None, None)

        if not self.mask_ffn:
            hidden_states = self.ffn(hidden_states)

        return hidden_states, attn_weights, current_key_value


class CpmBeeEncoder(nn.Module):
    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        self.num_layers = config.num_hidden_layers
        if config.mask_modules is not None:
            assert len(config.mask_modules) == self.num_layers, "The total number of masks should equal to num_layers"
            for mask_module in config.mask_modules:
                assert len(mask_module) == 2, "For encoder, each mask should be (mask_att, mask_ffn)"
        else:
            config.mask_modules = [(False, False)] * self.num_layers

        self.layers = nn.ModuleList(
            [
                CpmBeeTransformerBlock(
                    config, mask_att=config.mask_modules[ith][0], mask_ffn=config.mask_modules[ith][1]
                )
                for ith in range(self.num_layers)
            ]
        )

        self.output_layernorm = CpmBeeLayerNorm(config)

    def forward(
        self,
        hidden_states: torch.Tensor,
        attention_mask: torch.Tensor,
        position_bias: torch.Tensor,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        past_key_values: Optional[Tuple[torch.Tensor, torch.Tensor]] = None,
        use_cache: Optional[bool] = None,
    ):
        """
        Args:
            hidden_states (`torch.Tensor`):
                Input to the layer of shape `(batch, seq_len, dim_model)`
            attention_mask (`torch.Tensor`):
                Avoid invalid areas to participate in the calculation of shape `(batch, seq_len, seq_len)`
            position_bias (`torch.Tensor`):
                Provides position information to attention mechanism of shape `(num_heads, seq_len, seq_len)`
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers.
            output_hidden_states (`bool`, *optional*):
                Whether or not to return the hidden states of all layers.
            past_key_values (`Tuple[torch.Tensor, torch.Tensor])`, *optional*):
                Cached past key and value projection states
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding
                (see `past_key_values`).
        """
        all_hidden_states = () if output_hidden_states else None
        all_self_attns = () if output_attentions else None
        current_key_values = () if use_cache else None

        for i, layer in enumerate(self.layers):
            if output_hidden_states:
                all_hidden_states += (hidden_states,)
            layer_outputs = layer(
                hidden_states,
                attention_mask,
                position_bias,
                output_attentions=output_attentions,
                past_key_values=past_key_values[i] if past_key_values else None,
                use_cache=use_cache,
            )
            hidden_states, attn_weights, current_key_value = layer_outputs
            if output_attentions:
                all_self_attns += (attn_weights,)
            if current_key_value is not None:
                current_key_values = current_key_values + (current_key_value,)

        hidden_states = self.output_layernorm(hidden_states)

        if output_hidden_states:
            all_hidden_states += (hidden_states,)

        return hidden_states, current_key_values, all_hidden_states, all_self_attns


class CpmBeeBucketPositionBias(nn.Module):
    def __init__(self, config: CpmBeeConfig) -> None:
        super().__init__()

        self.num_heads = config.num_attention_heads
        self.num_buckets = config.position_bias_num_buckets
        self.num_segment_bucket = config.position_bias_num_segment_buckets
        self.max_distance = config.position_bias_max_distance

        self.relative_attention_bias = nn.Parameter(
            torch.empty(
                config.position_bias_num_buckets + config.position_bias_num_segment_buckets,
                config.num_attention_heads,
                dtype=config.torch_dtype,
            ),
        )

    def forward(self, query_pos: torch.Tensor, key_pos: torch.Tensor, rel_buckets: torch.Tensor):
        with torch.no_grad():
            batch = key_pos.size(0)
            keylen = key_pos.size(1)
            querylen = query_pos.size(1)

            if key_pos.size(0) != query_pos.size(0):
                raise AssertionError(
                    f"key_pos.size(0) should be equal to query_pos.size(0), but got {key_pos.size(0)} and {query_pos.size(0)}!"
                )
            if rel_buckets.size(0) != batch:
                raise AssertionError(
                    f"rel_buckets.size(0) should be equal to batch, but got {rel_buckets.size(0)} and {batch}!"
                )
            if rel_buckets.size(1) != querylen:
                raise AssertionError(
                    f"rel_buckets.size(1) should be equal to querylen, but got {rel_buckets.size(1)} and {querylen}!"
                )
            if rel_buckets.size(2) != keylen:
                raise AssertionError(
                    f"rel_buckets.size(2) should be equal to keylen, but got {rel_buckets.size(2)} and {keylen}!"
                )

            relative_position_bucket = rel_buckets - 1 + self.num_buckets

            inner_segment_bucket = self._position_bucket(
                key_pos[..., None, :] - query_pos[..., :, None],
                num_buckets=self.num_buckets,
                max_distance=self.max_distance,
            )
            relative_position_bucket = torch.where(
                rel_buckets == 0,
                inner_segment_bucket,
                relative_position_bucket,
            )

        embeds = nn.functional.embedding(relative_position_bucket, self.relative_attention_bias)
        embeds = embeds.permute(0, 3, 1, 2).contiguous()
        return embeds

    def _position_bucket(self, relative_position, num_buckets=32, max_distance=128):
        relative_buckets = 0
        num_buckets //= 2
        relative_buckets = (relative_position > 0).to(torch.int32) * num_buckets
        relative_position = torch.abs(relative_position)
        max_exact = num_buckets // 2
        is_small = relative_position < max_exact
        relative_postion_if_large = max_exact + (
            torch.log(relative_position.float() / max_exact)
            / math.log(max_distance / max_exact)
            * (num_buckets - max_exact)
        ).to(torch.int32)
        relative_postion_if_large = torch.min(
            relative_postion_if_large,
            torch.full_like(relative_postion_if_large, num_buckets - 1),
        )
        relative_buckets += torch.where(is_small, relative_position.to(torch.int32), relative_postion_if_large)
        return relative_buckets


# Copied from transformers.models.bert.modeling_bert.BertOutput with Bert->CPMBee
class CpmBeeOutput(nn.Module):
    def __init__(self, config):
        super().__init__()
        self.dense = nn.Linear(config.intermediate_size, config.hidden_size)
        self.LayerNorm = nn.LayerNorm(config.hidden_size, eps=config.layer_norm_eps)
        self.dropout = nn.Dropout(config.hidden_dropout_prob)

    def forward(self, hidden_states: torch.Tensor, input_tensor: torch.Tensor) -> torch.Tensor:
        hidden_states = self.dense(hidden_states)
        hidden_states = self.dropout(hidden_states)
        hidden_states = self.LayerNorm(hidden_states + input_tensor)
        return hidden_states


class CpmBeeRotaryEmbedding(nn.Module):
    """
    RotaryEmbedding embeds the unk token and special token. It will embeds the "...<mask>...<mask>...<unk>...<unk>..."
    to "...<mask_0>...<mask_1>...<unk_0>...<unk_1>..."" to help model to specify different special tokens and unk
    tokens.
    """

    def __init__(self, config: CpmBeeConfig):
        super().__init__()
        inv_freq = 1.0 / (10000 ** (torch.arange(0, config.hidden_size, 2, dtype=torch.float32) / config.hidden_size))
        self.distance_scale = config.distance_scale
        self.dtype = config.torch_dtype
        self.inv_freq = inv_freq.to(config.torch_dtype)

    def forward(self, x: torch.Tensor, x_pos: torch.Tensor):
        inv_freq = self.inv_freq.to(device=x.device, dtype=self.dtype)

        x_pos = x_pos * self.distance_scale
        freqs = x_pos[..., None].to(self.dtype) * inv_freq[None, :]  # (..., dim/2)

        emb = torch.cat((freqs, freqs), dim=-1)  # (..., dim)
        emb_cos = emb.cos()  # (..., dim)
        emb_sin = emb.sin()  # (..., dim)

        rotate_x = torch.cat([-x[..., x.size(-1) // 2 :], x[..., : x.size(-1) // 2]], dim=-1)  # (..., dim)

        return x * emb_cos + rotate_x * emb_sin


class CpmBeeEmbeddingExt(nn.Embedding):
    """
    Contains a RotaryEmbedding.
    """

    def __init__(self, config: CpmBeeConfig):
        super().__init__(config.vocab_size, config.hidden_size, dtype=config.torch_dtype)
        self.dim_model = config.hidden_size
        self.rotary_emb = CpmBeeRotaryEmbedding(config)

    def forward(self, ids: torch.Tensor, ids_sub: torch.Tensor):
        embeds = super().forward(ids) / math.sqrt(self.dim_model)
        return self.rotary_emb(embeds, ids_sub)

    def projection(self, x: torch.Tensor, ext_table: Optional[torch.Tensor] = None):
        logits = nn.functional.linear(x / math.sqrt(self.dim_model), self.weight)
        if ext_table is not None:
            logits_ext = nn.functional.linear(x, ext_table)
            logits = torch.cat([logits, logits_ext], dim=-1)
        return logits


class CpmBeePreTrainedModel(PreTrainedModel):
    """
    An abstract class to handle weights initialization and a simple interface for downloading and loading pretrained
    models.
    """

    config_class = CpmBeeConfig
    base_model_prefix = "cpmbee"
    supports_gradient_checkpointing = True
    _keys_to_ignore_on_load_missing = [r"position_ids"]

    def _init_weights(self, module):
        """Initialize the weights"""
        if isinstance(module, nn.Linear):
            module.weight.data.normal_(mean=0.0, std=self.config.init_std)
            if module.bias is not None:
                module.bias.data.zero_()
        # still needed
        elif isinstance(module, CpmBeeEmbeddingExt):
            module.weight.data.normal_(mean=0.0, std=self.config.init_std)
        elif isinstance(module, nn.LayerNorm):
            module.bias.data.zero_()
            module.weight.data.fill_(1.0)
        elif isinstance(module, CpmBeeLayerNorm):
            module.weight.data.fill_(1.0)
        elif isinstance(module, CpmBeeBucketPositionBias):
            module.relative_attention_bias.data.normal_(mean=0.0, std=self.config.init_std)

    def _set_gradient_checkpointing(self, module, value=False):
        if isinstance(module, CpmBeeEncoder):
            module.gradient_checkpointing = value


CPMBEE_START_DOCSTRING = r"""
    This model is a PyTorch [torch.nn.Module](https://pytorch.org/docs/stable/nn.html#torch.nn.Module) sub-class. Use
    it as a regular PyTorch Module and refer to the PyTorch documentation for all matter related to general usage and
    behavior.

    Parameters
        config ([`~CpmBeeConfig`]): Model configuration class with all the parameters of the
            Initializing with a config file does not load the weights associated with the model, only the
            configuration. Check out the [`~PreTrainedModel.from_pretrained`] method to load the model weights.
"""

CPMBEE_INPUTS_DOCSTRING = r"""
    Args:
        input_ids (`torch.Tensor` of shape `(batch_size, seq_len)`):
            Indices of input sequence tokens in the vocabulary.

            Indices can be obtained using [`CPMBeeTokenizer`]. See [`PreTrainedTokenizer.encode`] and
            [`PreTrainedTokenizer.__call__`] for details.

            [What are input IDs?](../glossary#input-ids)
        input_id_sub (`torch.Tensor` of shape `(batch_size, seq_len)`):
            Subscription of input sequence tokens in the vocabulary.

            Subscription of normal text will be zero while the special tokens of each group will be the 0, 1, 2, ...
            <ans_0>, <ans_1>, <ans_2> ... belongs to group <ans>. <mask_0>, <mask_1>, <mask_2> ... belongs to group
            <mask>.
        position (`torch.Tensor` of shape `(batch_size, seq_len)`):
            The position of input sequence tokens in the vocabulary for each segment. if segment1 is 0, 1, 2 and
            segment2 is 0, 1, 2, 3, the position will be 0, 1, 2, 0, 1, 2, 3
        context (`torch.Tensor` of shape `(batch_size, seq_len)`):
            Whether this token id is context or not. If is context, the value is 1. If not, the value is 0. If a token
            id is context, it does not need to be predicted.
        sample_ids (`torch.Tensor` of shape `(batch_size, seq_len)`):
            Give a sample id to every token id. The token ids with same sample ids belongs to the same sample.
        num_segments (`torch.Tensor` of shape `(batch_size, seq_len)`):
            Total number of segments in the current input.
        segment (`torch.Tensor` of shape `(batch_size, seq_len)`):
            Give a segment id to every token id. The token ids with same segment ids belongs to the same sample.

            Generally, a string key or value in input data will be a segment. For example, input {"input": "hello, ",
            "<ans>": ""}, the segments includes: "input", "hello, ", "<ans>" and "".
        segment_rel_offset (`torch.Tensor` of shape `(batch_size, seq_len)`):
            The offset of segment rel.
        segment_rel (`torch.Tensor` of shape `(batch_size, seq_len)`):
            The segment relevance. A relative implementation of measuring the importance of segments.
        past_states (`Dict[str, Union[torch.Tensor, List]]`):
            Store the history information including position, context, sample_ids, num_segments, segment and
            past_key_values.
        output_attentions (`bool`, *optional*):
            Whether or not to return the attentions tensors of all attention layers.
        output_hidden_states (`bool`, *optional*):
            Whether or not to return the hidden states of all layers.
        past_key_values (`tuple(tuple(torch.FloatTensor))`, *optional*, returned when `use_cache=True` is passed or when `config.use_cache=True`):
            A dummy arguments for CPMBee. The `past_states` contains pre-computed hidden-states (key and values in the
            self-attention blocks and in the cross-attention blocks) that can be used (see `past_key_values` input) and
            other history arguments to speed up sequential decoding.
        use_cache (`bool`, *optional*):
            If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding (see
            `past_key_values`).
        labels (`torch.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
            Labels for computing the masked language modeling loss.
        return_dict (`bool`, *optional*):
            Whether or not to return a [`~utils.ModelOutput`] instead of a plain tuple.
"""


@add_start_docstrings(
    "The bare CPMBee Model outputting raw hidden-states without any specific head on top.",
    CPMBEE_START_DOCSTRING,
)
class CpmBeeModel(CpmBeePreTrainedModel):
    def __init__(self, config: CpmBeeConfig):
        super().__init__(config)
        if config.half:
            config.torch_dtype = torch.half
        else:
            config.torch_dtype = torch.float
        self.encoder = CpmBeeEncoder(config)
        self.input_embedding = CpmBeeEmbeddingExt(config)
        self.position_bias = CpmBeeBucketPositionBias(config)
        self.vocab_size = config.vocab_size
        self.post_init()

    def get_input_embeddings(self):
        return self.input_embedding

    def set_input_embeddings(self, embeddings, **kwargs):
        self.input_embedding = embeddings

    @add_start_docstrings_to_model_forward(CPMBEE_INPUTS_DOCSTRING)
    @add_code_sample_docstrings(
        checkpoint=_CHECKPOINT_FOR_DOC,
        output_type=BaseModelOutputWithPast,
        config_class=_CONFIG_FOR_DOC,
    )
    def forward(
        self,
        input_ids: torch.Tensor,
        input_id_sub: Optional[torch.Tensor] = None,
        position: Optional[torch.Tensor] = None,
        context: Optional[torch.Tensor] = None,
        sample_ids: Optional[torch.Tensor] = None,
        num_segments: Optional[torch.Tensor] = None,
        segment: Optional[torch.Tensor] = None,
        segment_rel_offset: Optional[torch.Tensor] = None,
        segment_rel: Optional[torch.Tensor] = None,
        past_states: Optional[Dict] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        past_key_values: Optional[List] = None,
        use_cache: Optional[bool] = None,
        return_dict: Optional[bool] = None,
        **kwargs,
    ):
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict
        use_cache = use_cache if use_cache is not None else self.config.use_cache

        # dummy setting for common tests
        if input_id_sub is None:
            dtype, device = input_ids.dtype, input_ids.device
            batch, seq_length = input_ids.size()
            segment = torch.where(input_ids != 0, 2, 0).to(dtype=dtype, device=device)
            context = torch.full((batch, seq_length), 1, dtype=dtype, device=device)
            position = torch.arange(seq_length, dtype=dtype, device=device).repeat(batch, 1)
            input_id_sub = torch.full((batch, seq_length), 0, dtype=dtype, device=device)
            segment_rel_offset = torch.full((batch, seq_length), 0, dtype=dtype, device=device)
            segment_rel = torch.full((batch, seq_length), 0, dtype=dtype, device=device)
            num_segments = torch.full((batch, seq_length), 0, dtype=dtype, device=device)
            sample_ids = torch.zeros_like(input_ids)

        with torch.no_grad():
            if past_states is None:
                present_position = position
                present_context = context
                present_sample_ids = sample_ids
                present_num_segments = num_segments
                present_segments = segment
                present_buffer = None
            else:
                present_position = torch.cat([past_states["buffer_position"], position], dim=-1)
                present_context = torch.cat([past_states["buffer_context"], context], dim=-1)
                present_sample_ids = torch.cat([past_states["buffer_sample_ids"], sample_ids], dim=-1)
                present_num_segments = torch.cat([past_states["buffer_num_segments"], num_segments], dim=-1)
                present_segments = torch.cat([past_states["buffer_segments"], segment], dim=-1)
                present_buffer = past_states["buffer"]

            batch = input_ids.size(0)
            len_q = input_ids.size(1)
            len_buffer = present_position.size(1)

            segment_rel_2d = torch.masked_fill(
                segment[:, :, None] * num_segments[:, :, None]
                + present_segments[:, None, :]
                + segment_rel_offset[:, :, None],
                ~((sample_ids[:, :, None] == present_sample_ids[:, None, :])),  # not in the same sample
                0,  # avoid torch.gather overflow
            ).view(batch, len_q * len_buffer)

            segment_bucket = torch.gather(
                input=segment_rel,
                dim=1,
                index=segment_rel_2d.long(),
            ).view(batch, len_q, len_buffer)

            segment_bucket.masked_fill_(
                ~((sample_ids[:, :, None] == present_sample_ids[:, None, :])),  # not in the same span or sample
                1,  # bucket is used for in-context samples
            )

            # directional mask
            directional_mask_2d = present_position[:, None, :] <= position[:, :, None]
            # sample mask
            sample_mask_2d = (sample_ids[:, :, None] == 0) | (sample_ids[:, :, None] == present_sample_ids[:, None, :])
            # context mask
            attention_mask = present_context[:, None, :] | (
                context[:, :, None].logical_not() & directional_mask_2d.view(batch, len_q, len_buffer)
            )
            # span mask
            attention_mask = attention_mask & sample_mask_2d
            # length mask
            mask_1d = present_num_segments != 0
            attention_mask = mask_1d.view(batch, 1, len_buffer) & attention_mask

        hidden_states = self.input_embedding(input_ids, input_id_sub)
        position_bias = self.position_bias(position, present_position, segment_bucket)
        hidden_states, present_key_values, all_hidden_states, all_attentions = self.encoder(
            hidden_states,
            attention_mask,
            position_bias,
            output_attentions,
            output_hidden_states,
            present_buffer,
            use_cache,
        )

        if not return_dict:
            return tuple(
                v for v in [hidden_states, present_key_values, all_hidden_states, all_attentions] if v is not None
            )

        return BaseModelOutputWithPast(
            last_hidden_state=hidden_states,
            past_key_values=present_key_values,
            hidden_states=all_hidden_states,
            attentions=all_attentions,
        )


class CpmBeeBeamHypotheses(BeamHypotheses):
    def __init__(self, num_beams: int, length_penalty: float, early_stopping: bool, max_length: Optional[int] = None):
        """
        Override BeamHypotheses for CpmBee. The hyp to add is list but not tensor.
        """
        super().__init__(num_beams, length_penalty, early_stopping, max_length)

    def add(self, hyp: List, sum_logprobs: float, beam_indices: Optional[torch.LongTensor] = None):
        """
        Add a new hypothesis to the list.
        """
        score = sum_logprobs / (len(hyp) ** self.length_penalty)
        if len(self) < self.num_beams or score > self.worst_score:
            self.beams.append((score, hyp, beam_indices))
            if len(self) > self.num_beams:
                sorted_next_scores = sorted([(s, idx) for idx, (s, _, _) in enumerate(self.beams)])
                del self.beams[sorted_next_scores[0][1]]
                self.worst_score = sorted_next_scores[1][0]
            else:
                self.worst_score = min(score, self.worst_score)


class CPMBeeTransBlock(torch.nn.Module):
    def __init__(
        self,
        dim_model=4096,
        dim_ff=1024,
        dim_out=768,
        dtype=torch.float,
        eps=1e-6,
        dropout_p=0,
    ):
        super().__init__()
        if dropout_p is not None:
            self.dropout = torch.nn.Dropout(dropout_p)
        else:
            self.dropout = None
        self.w_out_res = torch.nn.Linear(dim_model, dim_out, bias=False)
        self.layernorm = torch.nn.LayerNorm(
            dim_out,
            dtype=dtype,
            eps=eps,
        )

    def forward(self, hidden_states: torch.Tensor):
        x_res = self.w_out_res(hidden_states)
        if self.dropout is not None:
            x_res = self.dropout(x_res)
        hidden_states = self.layernorm(x_res)
        return hidden_states


class CpmBeeWithTransform(CpmBeePreTrainedModel):
    _keys_to_ignore_on_load_missing = [r"lm_head.weight"]

    def __init__(self, config: CpmBeeConfig):
        super().__init__(config)
        self.llm = CpmBeeModel(config)

        self.trans_block = CPMBeeTransBlock(config.hidden_size, config.hidden_size // 4, config.unet_cross_attention_dim)

    def forward(
        self,
        input_ids: torch.Tensor,
        input_id_sub: Optional[torch.Tensor] = None,
        position: Optional[torch.Tensor] = None,
        context: Optional[torch.Tensor] = None,
        sample_ids: Optional[torch.Tensor] = None,
        num_segments: Optional[torch.Tensor] = None,
        segment: Optional[torch.Tensor] = None,
        segment_rel_offset: Optional[torch.Tensor] = None,
        segment_rel: Optional[torch.Tensor] = None,
        past_states: Optional[Dict] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        past_key_values: Optional[List] = None,
        use_cache: Optional[bool] = None,
        return_dict: Optional[bool] = None,
        **kwargs,
    ):
        outputs = self.llm(input_ids, input_id_sub, position, context,
            sample_ids, num_segments, segment, segment_rel_offset,
            segment_rel, past_states, output_attentions, output_hidden_states,
            past_key_values, use_cache, return_dict, **kwargs,)
        if return_dict:
            hidden_states = outputs.last_hidden_state
        else:
            hidden_states = outputs[0]
        #if self.trans_block is not None:
        #    hidden_states = self.trans_block(hidden_states)
        return outputs, hidden_states



def UNetSDModel(unet, adapter, replace=False):
    if replace:
        unet.forward = unet.forward_

    unet.forward_ = unet.forward

    def forward(self, sample, timestep, encoder_hidden_states, **kwargs):
        encoder_hidden_states = self.adapter(encoder_hidden_states)
        return unet.forward_(sample, timestep, encoder_hidden_states, **kwargs)

    def func(*args, **kwargs): return forward(unet, *args, **kwargs)
    unet.adapter = adapter
    unet.forward = func
    return unet


def UNetSDXLModel(unet, adapter, replace=False):
    # if replace:
    #     unet.forward = unet.forward_

    # unet.forward_ = unet.forward

    # def forward(self, sample, timestep, encoder_hidden_states, **kwargs):
    #     encoder_hidden_states, text_embeds = self.adapter(encoder_hidden_states, kwargs['added_cond_kwargs']['text_embeds'])
    #     kwargs['added_cond_kwargs']['text_embeds'] = text_embeds
    #     return unet.forward_(sample, timestep, encoder_hidden_states, **kwargs)

    # def func(*args, **kwargs): return forward(unet, *args, **kwargs)
    unet.adapter = adapter
    # unet.forward = func
    return unet



# --------------------------------------------------------
# InternVL
# Copyright (c) 2023 OpenGVLab
# Licensed under The MIT License [see LICENSE for details]
# --------------------------------------------------------

import torch
import torch.nn as nn
import torchvision.transforms as T
from torchvision.transforms import InterpolationMode
from transformers import LlamaTokenizer

from .configuration_intern_vit import InternVisionConfig
from .configuration_internvl import InternVLConfig
from .modeling_intern_vit import InternVisionModel
from .modeling_internvl import InternVL_C, InternVL_G, InternVLModel

__all__ = ['InternVisionConfig', 'InternVisionModel', 'InternVLConfig',
           'InternVLModel', 'InternVL_C', 'InternVL_G']


# Prefix the text "summarize:"
class InternVLTokenizer(nn.Module):
    def __init__(self, model_path):
        super(InternVLTokenizer, self).__init__()
        self.tokenizer = LlamaTokenizer.from_pretrained(model_path)
        self.tokenizer.pad_token = ' '  # allow padding
        self.tokenizer.add_eos_token = True

    def forward(self, text, prefix='summarize:'):
        if type(text) == str:
            text = prefix + text
        elif type(text) == list:
            text = [prefix + item for item in text]
        text = self.tokenizer(text, return_tensors='pt', max_length=80, truncation=True, padding='max_length').input_ids
        return text


def build_transform(task, image_size=224, mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]):
    if task == 'retrieval':
        transform = T.Compose([
            T.Lambda(lambda img: img.convert('RGB') if img.mode != 'RGB' else img),
            T.Resize((image_size, image_size), interpolation=InterpolationMode.BICUBIC),
            T.ToTensor(),
            T.Normalize(mean=mean, std=std)])
    else:
        transform = T.Compose([
            T.Lambda(lambda img: img.convert('RGB') if img.mode != 'RGB' else img),
            T.Resize(image_size, interpolation=InterpolationMode.BICUBIC),
            T.CenterCrop(image_size),
            T.ToTensor(),
            T.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225])])
    return transform


def load_internvl_c_huggingface(ckpt_path, device, task):
    model = InternVL_C.from_pretrained(ckpt_path, torch_dtype=torch.float16).to(device)
    if model.config.use_backbone_lora:
        model.vision_model.merge_and_unload()
        model.vision_model = model.vision_model.model
    if model.config.use_qllama_lora:
        model.qllama.merge_and_unload()
        model.qllama = model.qllama.model
    if model.config.force_image_size is not None:
        image_size = model.config.force_image_size
    else:
        image_size = model.config.vision_config.image_size
    transform = build_transform(task, image_size)
    tokenizer = InternVLTokenizer(ckpt_path)
    return model, transform, tokenizer


def load_internvl_g_huggingface(ckpt_path, device, task):
    model = InternVL_G.from_pretrained(ckpt_path, torch_dtype=torch.float16).to(device)
    if model.config.use_backbone_lora:
        model.vision_model.merge_and_unload()
        model.vision_model = model.vision_model.model
    if model.config.use_qllama_lora:
        model.qllama.merge_and_unload()
        model.qllama = model.qllama.model
    if model.config.force_image_size is not None:
        image_size = model.config.force_image_size
    else:
        image_size = model.config.vision_config.image_size
    transform = build_transform(task, image_size)
    tokenizer = InternVLTokenizer(ckpt_path)
    return model, transform, tokenizer



# --------------------------------------------------------
# InternVL
# Copyright (c) 2023 OpenGVLab
# Licensed under The MIT License [see LICENSE for details]
# --------------------------------------------------------
import copy

from transformers import LlamaConfig
from transformers.configuration_utils import PretrainedConfig
from transformers.utils import logging

from .configuration_intern_vit import InternVisionConfig

logger = logging.get_logger(__name__)


class InternVLConfig(PretrainedConfig):
    r"""
    [`InternVLConfig`] is the configuration class to store the configuration of a
    [`InternVLModel`]. It is used to instantiate a InternVLModel according to the specified
    arguments, defining the InternViT-6B and QLLaMA configs. Instantiating a configuration with
    the defaults will yield a similar configuration to that of the InternVL architecture.

    Configuration objects inherit from [`PretrainedConfig`] and can be used to control the model outputs. Read the
    documentation from [`PretrainedConfig`] for more information.

    Args:
        vision_config (`dict`, *optional*):
            Dictionary of configuration options used to initialize [`InternVisionConfig`].
        qllama_config (`dict`, *optional*):
            Dictionary of configuration options used to initialize [`LLaMAConfig`].
        clip_embed_dim (`int`, *optional*, defaults to 768):
            Size of the embeddings from the CLIP model.
        attn_pool_num_heads (`int`, *optional*, defaults to 16):
            Number of attention heads used in the attention pooling layers.
        num_query_token (`int`, *optional*, defaults to 96):
            Number of query tokens used in the transformer.
        label_smoothing (`float`, *optional*, defaults to 0.0):
            The amount of label smoothing to apply.
        cross_attention_frequency (`int`, *optional*, defaults to 2):
            The frequency of cross-attention layers in the model.
        use_backbone_lora (`int`, *optional*, defaults to 0):
            If non-zero, indicates the use of LoRA in the backbone of the model.
        use_qllama_lora (`int`, *optional*, defaults to 0):
            If non-zero, indicates the use of LoRA in the QLLaMA of the model.
        force_image_size (`int` or `None`, *optional*):
            If not None, forces the model to use this specific image size.
        initializer_range (`float`, *optional*, defaults to 0.02):
            The standard deviation of the truncated_normal_initializer for initializing all weight matrices.
        kwargs (*optional*):
            Dictionary of additional keyword arguments.
    """

    model_type = 'internvl'
    is_composition = True

    def __init__(
            self,
            vision_config=None,
            qllama_config=None,
            clip_embed_dim=768,
            attn_pool_num_heads=16,
            num_query_token=96,
            label_smoothing=0.0,
            cross_attention_frequency=2,
            use_backbone_lora=0,
            use_qllama_lora=0,
            force_image_size=None,
            initializer_range=0.02,
            **kwargs):
        super().__init__(**kwargs)

        if vision_config is None:
            vision_config = {}
            logger.info('vision_config is None. initializing the InternVisionConfig with default values.')

        if qllama_config is None:
            qllama_config = {}
            logger.info(
                'qllama_config is None. Initializing the InternTextConfig config with default values (`LlamaConfig`).')

        self.vision_config = InternVisionConfig(**vision_config)
        self.qllama_config = LlamaConfig(**qllama_config)
        self.qllama_config.num_query_token = num_query_token
        self.qllama_config.cross_attention_frequency = cross_attention_frequency
        self.hidden_size = self.qllama_config.hidden_size

        self.clip_embed_dim = clip_embed_dim
        self.attn_pool_num_heads = attn_pool_num_heads
        self.num_query_token = num_query_token
        self.label_smoothing = label_smoothing
        self.use_backbone_lora = use_backbone_lora
        self.use_qllama_lora = use_qllama_lora
        self.force_image_size = force_image_size
        self.initializer_range = initializer_range

    def to_dict(self):
        """
        Serializes this instance to a Python dictionary. Override the default [`~PretrainedConfig.to_dict`].

        Returns:
            `Dict[str, any]`: Dictionary of all the attributes that make up this configuration instance,
        """
        output = copy.deepcopy(self.__dict__)
        output['vision_config'] = self.vision_config.to_dict()
        output['qllama_config'] = self.qllama_config.to_dict()
        output['model_type'] = self.__class__.model_type
        return output



import os

import torch
from diffusers import PixArtAlphaPipeline, StableDiffusionPipeline
from huggingface_hub import hf_hub_download
from transformers import AutoTokenizer

from .internvl.modeling_intern_text import InternVLTextModel
from .models.adapter import LanguageAdapter, LanguageAdapterXL, TextProjection
from .models.unet import UNetSDModel, UNetSDXLModel
from .patch import (
    MAX_TOKEN_LENGTH,
    encode_prompt_pixart,
    encode_prompt_sd15,
    encode_prompt_sdxl,
    setup,
)

SD15_PIPELINES = [
    StableDiffusionPipeline,
]
PIXART_PIPELINES = [
    PixArtAlphaPipeline,
]
SDXL_PIPELINES = []


def transform(
    pipe,
    adapter_path=None,
    adapter=None,
    text_encoder_path='OpenGVLab/InternVL-14B-224px',
    text_encoder=None,
    tokenizer=None,
    pipe_type=None,
    replace=False,
):
    if pipe_type is None:
        pipe_type = infer_pipe_type(pipe, adapter_path, adapter)

    text_encoder, tokenizer = load_internvl(text_encoder_path, text_encoder, tokenizer, torch_dtype=pipe.text_encoder.dtype)

    tokenizer.pad_token_id = 0  # set pad_token_id to 0
    tokenizer.model_max_length = MAX_TOKEN_LENGTH
    pipe.text_encoder = text_encoder.to(pipe.text_encoder.device)
    pipe.tokenizer = tokenizer

    if adapter is None:
        adapter = load_adapter(adapter_path, type=pipe_type)

    if pipe_type in ['sd15', 'sd21', 'sdxl']:
        adapter = adapter.to(pipe.text_encoder.device)
        if pipe_type in ['sd15', 'sd21']:
            def func(*args, **kwargs): return encode_prompt_sd15(pipe, *args, **kwargs)
            unet = UNetSDModel(pipe.unet, adapter, replace=replace)
            pipe.unet = unet.to(device=pipe.unet.device, dtype=pipe.unet.dtype)
        elif pipe_type == 'sdxl':
            def func(*args, **kwargs): return encode_prompt_sdxl(pipe, *args, **kwargs)
            unet = UNetSDXLModel(pipe.unet, adapter, replace=replace)
            if adapter.add_embedding is not None:
                print('load add embedding')
                unet.add_embedding = adapter.add_embedding
            pipe.unet = unet.to(device=pipe.unet.device, dtype=pipe.unet.dtype)
    elif pipe_type == 'pixart':
        def func(*args, **kwargs): return encode_prompt_pixart(pipe, *args, **kwargs)
        if adapter is not None:
            pipe.transformer.caption_projection = adapter.to(device=pipe.text_encoder.device, dtype=pipe.text_encoder.dtype)
    pipe.encode_prompt = func

    print('pipe_type', pipe_type)
    return pipe


def load_internvl(model_id, text_encoder, tokenizer, torch_dtype=None):
    if text_encoder is None:
        text_encoder = InternVLTextModel.from_pretrained(
            model_id,
            low_cpu_mem_usage=True,
            torch_dtype=torch_dtype,
        )
    if tokenizer is None:
        tokenizer = AutoTokenizer.from_pretrained(model_id, use_fast=False, add_eos_token=True)

    return text_encoder, tokenizer


def load_adapter(path, device=None, type=None):
    if path is None:
        return None

    if type is None:
        type = infer_pipe_type(None, path)
    print(type, flush=True)
    
    if not os.path.exists(path):
        try:
            repo_id, filename = path.split('::')
            path = hf_hub_download(repo_id=repo_id, filename=filename)
        except Exception as e:
            raise ValueError(f'{path} not found')
        
    adapter_state_dict = torch.load(path)

    if type in ['sd15', 'sd21']:
        adapter = LanguageAdapter(4096, 768 if type == 'sd15' else 1024, num_queries=77 if 'direct' in path else 120)
    elif type == 'pixart':
        adapter = TextProjection(4096, 1152)
    elif type == 'sdxl':
        adapter = LanguageAdapterXL(
            4096,
            require_text_embedding=any('add_embedding' in key for key in adapter_state_dict.keys()),
            attention_pooling=any('positional_embedding' in key for key in adapter_state_dict.keys()),
        )

    adapter.load_state_dict(adapter_state_dict)
    if device is not None:
        adapter = adapter.to(device)
    adapter.pipe_type = type
    return adapter


def infer_pipe_type(pipe, adapter_path, adapter=None):
    if adapter is not None:
        return adapter.pipe_type

    if adapter_path is None:
        adapter_path = ''

    if 'sd15' in adapter_path:
        return 'sd15'
    if 'sd21' in adapter_path:
        return 'sd21'
    if any([isinstance(pipe, cls) for cls in SD15_PIPELINES]):
        if 'sd15' in adapter_path:
            return 'sd15'
        if 'sd21' in adapter_path:
            return 'sd21'
    if any([isinstance(pipe, cls) for cls in SDXL_PIPELINES]) or 'sdxl' in adapter_path:
        return 'sdxl'
    if any([isinstance(pipe, cls) for cls in PIXART_PIPELINES]) or 'pixart' in adapter_path:
        return 'pixart'
    raise ValueError('Could not automaticly infer pipe type, please specify pipe_type')



from setuptools import setup, find_packages

deps = [
    'fire',
    'diffusers',
    'huggingface_hub',
    'transformers',
    'numpy',
    'gradio',
    'timm',
    
]

setup(
    name='mulankit',
    description='MuLan: Adapting Multilingual Diffusion Models for 110+ Languages.',
    url='https://github.com/mulanai/MuLan',
    author='Zeqiang Lai',
    author_email='laizeqiang@outlook.com',
    packages=find_packages(),
    version='1.0.1',
    include_package_data=True,
    install_requires=deps,
    package_data={
        'mulankit': [
            'app/style.css',
            'app/meta.json',
            'app/samples.txt',
        ],
    },
)



import math
import fire
import os
import random
import uuid
from datetime import datetime
from typing import Tuple
import traceback
import json

import gradio as gr
import numpy as np
import torch
from diffusers import (
    AutoPipelineForText2Image,
    StableDiffusionPipeline,
    StableDiffusionXLPipeline,
    DPMSolverMultistepScheduler,
    LCMScheduler,
    AutoencoderKL,
    PixArtAlphaPipeline,
    DiffusionPipeline,
    Transformer2DModel,
    EulerDiscreteScheduler,
)
from transformers import AutoTokenizer
from huggingface_hub import hf_hub_download

import mulankit
from mulankit.internvl.modeling_intern_text import InternVLTextModel
from mulankit.patch import (
    MAX_TOKEN_LENGTH,
    FORCE_SDXL_ZERO_POOL_PROMPT,
    FORCE_SDXL_ZERO_EMPTY_PROMPT,
    FORCE_SDXL_ZERO_NEGATIVE_POOL_PROMPT
)

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
CSS_PATH = os.environ.get('CSS_PATH', os.path.join(CURRENT_DIR, 'style.css'))
MODEL_CHOICES_PATH = os.environ.get('MODEL_CHOICES_PATH', os.path.join(CURRENT_DIR, 'meta.json'))
SAMPLE_CHOICES_PATH = os.environ.get('SAMPLE_CHOICES_PATH', os.path.join(CURRENT_DIR, 'samples.txt'))
INTERNVL_PATH = os.environ.get('INTERNVL_PATH', 'OpenGVLab/InternVL-14B-224px')
LOCAL_FILES = bool(os.environ.get('LOCAL_FILES', False))
SAFTY_CHECKER = bool(os.environ.get('SAFTY_CHECKER', True))

MAX_SEED = np.iinfo(np.int32).max
CACHE_EXAMPLES = torch.cuda.is_available() and os.getenv("CACHE_EXAMPLES", "1") == "1"
MAX_IMAGE_SIZE = int(os.getenv("MAX_IMAGE_SIZE", "2048"))
ENABLE_CPU_OFFLOAD = os.getenv("ENABLE_CPU_OFFLOAD", "0") == "1"

device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
default_scheduler = None

DEFAULT_STYLE_NAME = "(No style)"
SCHEDULE_NAME = ["Default", "DPM-Solver", "LCM-Solver", "SDXL-Lightning", "SDXL-Lightning-1step"]
DEFAULT_SCHEDULE_NAME = "Default"
NUM_IMAGES_PER_PROMPT = 1


def apply_style(style_name: str, positive: str, negative: str = "") -> Tuple[str, str]:
    p, n = styles.get(style_name, styles[DEFAULT_STYLE_NAME])
    if not negative:
        negative = ""
    return p.replace("{prompt}", positive), n + negative


def load_file(path):
    if os.path.exists(path):
        return path
    repo, ckpt = path.split(':')
    path = hf_hub_download(repo, ckpt)
    return path


def load_model_auto_clean(models, model_category, pipe_path, adapter_path, vae_path, lora_path, delta_path):
    try:
        return load_model(models, model_category, pipe_path, adapter_path, vae_path, lora_path, delta_path)
    except Exception as e:
        error = traceback.format_exc()
        if isinstance(e, torch.cuda.OutOfMemoryError):
            print('error loading model, move all models to cpu and reload')
            for k, v in models.items():
                v['None'].to('cpu')
            return load_model(models, model_category, pipe_path, adapter_path, vae_path, lora_path, delta_path)
        else:
            raise e


def load_from_single_file(model_category, pipe_path):
    model_category = model_category.lower()
    pipe_path = load_file(pipe_path)
    if model_category in ['sd15', 'sd21']:
        pipe = StableDiffusionPipeline.from_single_file(pipe_path, torch_dtype=torch.float16)
    elif model_category in ['sdxl']:
        pipe = StableDiffusionXLPipeline.from_single_file(pipe_path, torch_dtype=torch.float16)
    else:
        raise ValueError('invalid path')
    return pipe


def load_special_pipe(pipe_path):
    if pipe_path == 'openbmb/VisCPM-Paint':
        from .viscpm.tokenization_viscpmbee import VisCpmBeeTokenizer
        from .viscpm.pipeline_viscpm_paint import VisCPMPaintBeePipeline
        from .viscpm.modeling_cpmbee import CpmBeeWithTransform
        tokenizer_ = VisCpmBeeTokenizer.from_pretrained(pipe_path, trust_remote_code=True)
        text_encoder_ = CpmBeeWithTransform.from_pretrained(pipe_path, trust_remote_code=True)
        pipe = VisCPMPaintBeePipeline.from_pretrained(pipe_path, text_encoder=text_encoder_, tokenizer=tokenizer_)
    elif pipe_path == 'BAAI/AltDiffusion-m18':
        from diffusers import AltDiffusionPipeline, UNet2DConditionModel, EulerAncestralDiscreteScheduler, AutoencoderKL
        from diffusers.pipelines.deprecated.alt_diffusion.modeling_roberta_series import RobertaSeriesModelWithTransformation
        from transformers import XLMRobertaTokenizer
        pipe = AltDiffusionPipeline(
            text_encoder=RobertaSeriesModelWithTransformation.from_pretrained(pipe_path, subfolder='text_encoder'),
            tokenizer=XLMRobertaTokenizer.from_pretrained(pipe_path, subfolder='tokenizer'),
            scheduler=EulerAncestralDiscreteScheduler.from_pretrained(pipe_path, subfolder='scheduler'),
            unet=UNet2DConditionModel.from_pretrained(pipe_path, subfolder='unet'),
            vae=AutoencoderKL.from_pretrained(pipe_path, subfolder='vae'),
            safety_checker=None,
            feature_extractor=None,
        )
        pipe = pipe.to(dtype=torch.float16)
    elif pipe_path == 'stabilityai/japanese-stable-diffusion-xl':
        pipe = DiffusionPipeline.from_pretrained("stabilityai/japanese-stable-diffusion-xl", trust_remote_code=True, torch_dtype=torch.float16)
    else:
        raise ValueError(f'could not load pipe {pipe_path}')
    return pipe


def load_delta(pipe, delta_path):
    print('load delta')
    if isinstance(pipe, PixArtAlphaPipeline):
        old_state_dict = pipe.transformer.state_dict()
        delta_state_dict = torch.load(delta_path, map_location='cpu')
        new_state_dict = {k: old_state_dict[k].to('cpu').float()+delta_state_dict[k].to('cpu').float()
                          for k in delta_state_dict.keys()}
        if 'adaln_single.emb.resolution_embedder.linear_1.weight' not in delta_state_dict.keys():
            dtype = pipe.transformer.dtype
            pipe.transformer = Transformer2DModel.from_config(pipe.transformer.config, sample_size=64)
            pipe.transformer.to(dtype)
        pipe.transformer.load_state_dict(new_state_dict)
    return pipe


def load_model(models, model_category, pipe_path, adapter_path, vae_path, lora_path, delta_path):
    gr.Info(f'Loading model {pipe_path} \n adapter {adapter_path}')
    print(models.keys(), flush=True)

    variant = None
    full_path = pipe_path
    if ':' in pipe_path:
        variant, pipe_path = pipe_path.split(':')

    # load pipe first
    if full_path not in models or adapter_path == 'None':
        if pipe_path.endswith('.safetensors'):
            pipe = load_from_single_file(model_category, pipe_path)
        else:
            try:
                pipe = AutoPipelineForText2Image.from_pretrained(
                    pipe_path,
                    variant=variant,
                    torch_dtype=torch.float16,
                    local_files_only=LOCAL_FILES
                )
            except Exception as e:
                pipe = load_special_pipe(pipe_path)
        if SAFTY_CHECKER:
            pipe.safety_checker = None
        models[full_path] = {'None': pipe.to(device)}

    pipe = models[full_path]['None']
    if vae_path != 'None':
        if 'vae' not in models:
            models['vae'] = {}
        if vae_path not in models['vae']:
            try:
                vae = AutoencoderKL.from_pretrained(vae_path, torch_dtype=torch.float16)
            except Exception as e:
                vae = AutoencoderKL.from_pretrained(vae_path, torch_dtype=torch.float16, subfolder='vae')
            models['vae'][vae_path] = vae
        vae = models['vae'][vae_path]
        pipe.vae = vae.to(pipe.vae.device)

    if delta_path != 'None':
        pipe = load_delta(pipe, delta_path)

    if adapter_path != 'None' or delta_path != 'None':
        pipe = mulankit.transform(
            pipe,
            adapter_path if adapter_path != 'None' else None,
            text_encoder=text_encoder,
            tokenizer=tokenizer,
            replace=full_path in models and len(models[full_path].keys()) > 1,
        )
        models[full_path][adapter_path] = pipe.to(device)

    if lora_path != 'None':
        pipe.load_lora_weights(load_file(lora_path))
        pipe.fuse_lora()

    global default_scheduler
    default_scheduler = pipe.scheduler

    gr.Info('Model loaded')
    print('loaded', flush=True)
    return models, gr.update(interactive=True)


def save_image(img):
    unique_name = str(uuid.uuid4()) + '.png'
    save_path = os.path.join(f'output/{datetime.now().date()}')
    os.makedirs(save_path, exist_ok=True)
    unique_name = os.path.join(save_path, unique_name)
    img.save(unique_name)
    return unique_name


def randomize_seed_fn(seed: int, randomize_seed: bool) -> int:
    if randomize_seed:
        seed = random.randint(0, MAX_SEED)
    return seed


def generate(
    prompt: str,
    negative_prompt: str = "",
    style: str = DEFAULT_STYLE_NAME,
    use_negative_prompt: bool = False,
    seed: int = 0,
    width: int = 1024,
    height: int = 1024,
    schedule: str = 'default',
    guidance_scale: float = 4.5,
    num_inference_steps: int = 20,
    randomize_seed: bool = False,
    num_of_image: int = 1,
    pipes=None,
    selected_model=None,
    selected_adapter=None,
    selected_vae=None,
    selected_lora=None,
    selected_delta=None,
    max_token_length=MAX_TOKEN_LENGTH,
    force_sdxl_zero_pool_prompt=FORCE_SDXL_ZERO_POOL_PROMPT,
    force_sdxl_zero_empty_prompt=FORCE_SDXL_ZERO_EMPTY_PROMPT,
    force_sdxl_zero_negative_pool_prompt=FORCE_SDXL_ZERO_NEGATIVE_POOL_PROMPT,
    use_resolution_binning: bool = True,
    upload_image=None,
    progress=gr.Progress(track_tqdm=True),
):
    pipe = pipes[selected_model][selected_adapter]

    seed = int(randomize_seed_fn(seed, randomize_seed))
    generator = torch.manual_seed(seed)

    if schedule == 'DPM-Solver':
        if not isinstance(pipe.scheduler, DPMSolverMultistepScheduler):
            pipe.scheduler = DPMSolverMultistepScheduler.from_config(pipe.scheduler.config)
    elif schedule == "LCM-Solver":
        if not isinstance(pipe.scheduler, LCMScheduler):
            pipe.scheduler = LCMScheduler.from_config(pipe.scheduler.config)
    elif schedule == 'SDXL-Lightning':
        pipe.scheduler = EulerDiscreteScheduler.from_config(pipe.scheduler.config, timestep_spacing="trailing")
    elif schedule == 'SDXL-Lightning-1step':
        pipe.scheduler = EulerDiscreteScheduler.from_config(pipe.scheduler.config, timestep_spacing="trailing", prediction_type="sample")
    else:
        pipe.scheduler = default_scheduler

    if not use_negative_prompt:
        negative_prompt = None  # type: ignore
    prompt, negative_prompt = apply_style(style, prompt, negative_prompt)

    if not use_negative_prompt:
        negative_prompt = ""

    mulankit.setup(
        max_token_length=max_token_length,
        force_sdxl_zero_pool_prompt=force_sdxl_zero_pool_prompt,
        force_sdxl_zero_empty_prompt=force_sdxl_zero_empty_prompt,
        force_sdxl_zero_negative_pool_prompt=force_sdxl_zero_negative_pool_prompt,
    )

    kwargs = {}
    if isinstance(pipe, PixArtAlphaPipeline):
        kwargs["use_resolution_binning"] = use_resolution_binning

    fail = 0
    while fail < 3:
        try:
            images = pipe(
                prompt=prompt,
                negative_prompt=negative_prompt,
                width=width,
                height=height,
                guidance_scale=guidance_scale,
                num_inference_steps=num_inference_steps,
                generator=generator,
                num_images_per_prompt=num_of_image,
                output_type="pil",
                **kwargs,
            ).images
            break
        except Exception as e:
            error = traceback.format_exc()
            print('error loading model, move all models to cpu and reload')
            for k, v in pipes.items():
                v['None'].to('cpu')
            pipe.to('cuda')
            fail += 1
            continue

    image_paths = [save_image(img) for img in images]
    return image_paths, seed


def load_model_list(category):
    category = category.lower()
    meta = json.load(open(MODEL_CHOICES_PATH, 'r'))[category]
    return meta['model'], meta['adapter'], meta['vae'], meta['lora'], meta.get('delta', ['None'])


def reload_model_list(category):
    model_list, adapter_list, vae_list, lora_list, delta_list = load_model_list(category)
    return (
        gr.update(choices=model_list, value=model_list[0]),
        gr.update(choices=adapter_list, value=adapter_list[0]),
        gr.update(choices=vae_list, value=vae_list[0]),
        gr.update(choices=lora_list, value=lora_list[0]),
        gr.update(choices=delta_list, value=delta_list[0])
    )


def put_images_to_history(outputs, state):
    if 'history' not in state:
        state['history'] = []
    state['history'] += outputs
    return state['history']


def main(
    ip='0.0.0.0',
    port=10024,
    share=False,
):
    print('launching demo', flush=True)

    DESCRIPTION = """
    # 🌻 MuLan: Adapting Multilingual Diffusion Models for 110 + Languages
    [[Technical Report (coming soon)]](#) [[Code]](https://github.com/mulanai/MuLan) [[Model]](https://huggingface.co/mulanai)
    """
    if not torch.cuda.is_available():
        DESCRIPTION += "\n<p>Running on CPU 🥶 This demo does not work on CPU.</p>"

    COPY_RIGHT = """
    <div align="center">
    Feel free to steal this gradio's source code. Templated by [MuLan]().
    </div>
    """

    examples = [line.strip() for line in open(SAMPLE_CHOICES_PATH, 'r').readlines()]

    MODEL_CHOICES, ADAPTER_CHOICES, VAE_CHOICES, LORA_CHOICES, DELTA_CHOICES = load_model_list(DEFAULT_CATEGORY)

    state_, _ = load_model({'history': []}, DEFAULT_CATEGORY, MODEL_CHOICES[0], ADAPTER_CHOICES[0], VAE_CHOICES[0], LORA_CHOICES[0], DELTA_CHOICES[0])

    with gr.Blocks(css=CSS_PATH) as demo:
        state = gr.State(state_)
        del state_

        gr.Markdown(DESCRIPTION)

        with gr.Row():
            with gr.Column():
                with gr.Tabs():
                    with gr.Tab("Models"):
                        model_category = gr.Radio(
                            show_label=True,
                            label="Model Category",
                            container=True,
                            interactive=True,
                            choices=CATEGORIES,
                            value=DEFAULT_CATEGORY,
                        )
                        with gr.Group():
                            selected_model = gr.Dropdown(choices=MODEL_CHOICES, value=MODEL_CHOICES[0], allow_custom_value=True, label='Model')
                            selected_adapter = gr.Dropdown(choices=ADAPTER_CHOICES, value=ADAPTER_CHOICES[0], allow_custom_value=True, label='Language Adapter')

                            with gr.Accordion('More', open=False):
                                selected_vae = gr.Dropdown(choices=VAE_CHOICES, value=VAE_CHOICES[0], allow_custom_value=True, label='VAE')
                                selected_lora = gr.Dropdown(choices=LORA_CHOICES, value=LORA_CHOICES[0], allow_custom_value=True, label='LoRA')
                                selected_delta = gr.Dropdown(choices=DELTA_CHOICES, value=DELTA_CHOICES[0], allow_custom_value=True, label='Delta')

                            with gr.Row():
                                reload_button = gr.Button('Reload List')
                                load_button = gr.Button('Load')
                    with gr.Tab("Image"):
                        upload_image = gr.Image()

                    with gr.Tab("History"):
                        history_image = gr.Gallery(show_label=False, columns=1)

            with gr.Column(scale=2):
                with gr.Group():
                    with gr.Row():
                        prompt = gr.Text(
                            label="Prompt",
                            show_label=False,
                            max_lines=1,
                            placeholder="Enter your prompt",
                            container=False,
                        )
                        run_button = gr.Button("Run", scale=0)
                    result = gr.Gallery(columns=NUM_IMAGES_PER_PROMPT, show_label=False, height=520, interactive=False)

            with gr.Column():
                with gr.Tabs():
                    with gr.Tab("Basic"):
                        inference_steps = gr.Slider(
                            label="inference steps",
                            minimum=1,
                            maximum=50,
                            step=1,
                            value=25,
                        )

                        with gr.Group():
                            resolution_bining = gr.Checkbox(label="Resolution Bining", value=True)
                            with gr.Row():
                                width = gr.Slider(
                                    label="Width",
                                    minimum=256,
                                    maximum=MAX_IMAGE_SIZE,
                                    step=32,
                                    value=512,
                                )
                                height = gr.Slider(
                                    label="Height",
                                    minimum=256,
                                    maximum=MAX_IMAGE_SIZE,
                                    step=32,
                                    value=512,
                                )
                            num_of_image = gr.Slider(
                                label="Number of Images",
                                minimum=1,
                                maximum=25,
                                step=1,
                                value=1,
                            )
                            seed = gr.Slider(
                                label="Seed",
                                minimum=0,
                                maximum=MAX_SEED,
                                step=1,
                                value=0,
                            )
                            randomize_seed = gr.Checkbox(label="Randomize seed", value=True)

                        with gr.Group():
                            use_negative_prompt = gr.Checkbox(label="Use Negative Prompt", value=False, visible=True)
                            negative_prompt = gr.Text(
                                max_lines=1,
                                placeholder="Enter a negative prompt",
                                visible=False,
                                show_label=False,
                            )
                    with gr.Tab("Style"):
                        style_selection = gr.Radio(
                            show_label=True,
                            container=True,
                            interactive=True,
                            choices=STYLE_NAMES,
                            value=DEFAULT_STYLE_NAME,
                            label="Image Style",
                        )
                    with gr.Tab("Scheduler"):
                        schedule = gr.Radio(
                            show_label=True,
                            container=True,
                            interactive=True,
                            choices=SCHEDULE_NAME,
                            value=DEFAULT_SCHEDULE_NAME,
                            label="Sampler Schedule",
                            visible=True,
                        )
                        guidance_scale = gr.Slider(
                            label="Guidance Scale",
                            minimum=0,
                            maximum=10,
                            step=0.1,
                            value=4.5,
                        )
                    with gr.Tab("Others"):
                        max_token_length = gr.Slider(
                            label="Max Token Length",
                            minimum=80,
                            maximum=500,
                            step=20,
                            value=MAX_TOKEN_LENGTH,
                        )
                        force_sdxl_zero_pool_prompt = gr.Checkbox(label="force_sdxl_zero_pool_prompt", value=FORCE_SDXL_ZERO_POOL_PROMPT)
                        force_sdxl_zero_empty_prompt = gr.Checkbox(label="force_sdxl_zero_empty_prompt", value=FORCE_SDXL_ZERO_EMPTY_PROMPT)
                        force_sdxl_zero_negative_pool_prompt = gr.Checkbox(label="force_sdxl_zero_negative_pool_prompt", value=FORCE_SDXL_ZERO_NEGATIVE_POOL_PROMPT)

        gr.Examples(
            examples=examples,
            inputs=prompt,
            outputs=[result, seed],
            fn=generate,
            examples_per_page=5,
        )

        # gr.Markdown(COPY_RIGHT)

        model_category.change(reload_model_list, inputs=[model_category], outputs=[selected_model, selected_adapter, selected_vae, selected_lora, selected_delta])

        use_negative_prompt.change(
            fn=lambda x: gr.update(visible=x),
            inputs=use_negative_prompt,
            outputs=negative_prompt,
        )

        num_of_image.change(
            fn=lambda x: gr.update(columns=int(math.sqrt(x))),
            inputs=num_of_image,
            outputs=result,
        )

        load_button.click(
            fn=lambda x: gr.update(interactive=False),
            inputs=run_button,
            outputs=run_button,
        ).then(
            fn=load_model_auto_clean,
            inputs=[state, model_category, selected_model, selected_adapter, selected_vae, selected_lora, selected_delta],
            outputs=[state, run_button],
        )

        reload_button.click(reload_model_list, inputs=[model_category], outputs=[selected_model, selected_adapter, selected_vae, selected_lora, selected_delta])

        gr.on(
            triggers=[
                prompt.submit,
                negative_prompt.submit,
                run_button.click,
            ],
            fn=generate,
            inputs=[
                prompt,
                negative_prompt,
                style_selection,
                use_negative_prompt,
                seed,
                width,
                height,
                schedule,
                guidance_scale,
                inference_steps,
                randomize_seed,
                num_of_image,
                state,
                selected_model,
                selected_adapter,
                selected_vae,
                selected_lora,
                selected_delta,
                max_token_length,
                force_sdxl_zero_pool_prompt,
                force_sdxl_zero_empty_prompt,
                force_sdxl_zero_negative_pool_prompt,
                resolution_bining,
                upload_image,
            ],
            outputs=[result, seed],
        ).then(
            put_images_to_history,
            inputs=[result, state], outputs=[history_image]
        )

    print('launched', flush=True)
    demo.launch(server_name=ip, server_port=port, share=share)


if __name__ == "__main__":
    text_encoder = InternVLTextModel.from_pretrained(INTERNVL_PATH, low_cpu_mem_usage=True, torch_dtype=torch.float16)
    tokenizer = AutoTokenizer.from_pretrained(INTERNVL_PATH, use_fast=False, add_eos_token=True)
    tokenizer.pad_token_id = 0  # set pad_token_id to 0
    tokenizer.model_max_length = 80

    style_list = [
        {
            "name": "(No style)",
            "prompt": "{prompt}",
            "negative_prompt": "",
        },
        {
            "name": "Cinematic",
            "prompt": "cinematic still {prompt} . emotional, harmonious, vignette, highly detailed, high budget, bokeh, cinemascope, moody, epic, gorgeous, film grain, grainy",
            "negative_prompt": "anime, cartoon, graphic, text, painting, crayon, graphite, abstract, glitch, deformed, mutated, ugly, disfigured",
        },
        {
            "name": "Photographic",
            "prompt": "cinematic photo {prompt} . 35mm photograph, film, bokeh, professional, 4k, highly detailed",
            "negative_prompt": "drawing, painting, crayon, sketch, graphite, impressionist, noisy, blurry, soft, deformed, ugly",
        },
        {
            "name": "Anime",
            "prompt": "anime artwork {prompt} . anime style, key visual, vibrant, studio anime,  highly detailed",
            "negative_prompt": "photo, deformed, black and white, realism, disfigured, low contrast",
        },
        {
            "name": "Manga",
            "prompt": "manga style {prompt} . vibrant, high-energy, detailed, iconic, Japanese comic style",
            "negative_prompt": "ugly, deformed, noisy, blurry, low contrast, realism, photorealistic, Western comic style",
        },
        {
            "name": "Digital Art",
            "prompt": "concept art {prompt} . digital artwork, illustrative, painterly, matte painting, highly detailed",
            "negative_prompt": "photo, photorealistic, realism, ugly",
        },
        {
            "name": "Pixel art",
            "prompt": "pixel-art {prompt} . low-res, blocky, pixel art style, 8-bit graphics",
            "negative_prompt": "sloppy, messy, blurry, noisy, highly detailed, ultra textured, photo, realistic",
        },
        {
            "name": "Fantasy art",
            "prompt": "ethereal fantasy concept art of  {prompt} . magnificent, celestial, ethereal, painterly, epic, majestic, magical, fantasy art, cover art, dreamy",
            "negative_prompt": "photographic, realistic, realism, 35mm film, dslr, cropped, frame, text, deformed, glitch, noise, noisy, off-center, deformed, cross-eyed, closed eyes, bad anatomy, ugly, disfigured, sloppy, duplicate, mutated, black and white",
        },
        {
            "name": "Neonpunk",
            "prompt": "neonpunk style {prompt} . cyberpunk, vaporwave, neon, vibes, vibrant, stunningly beautiful, crisp, detailed, sleek, ultramodern, magenta highlights, dark purple shadows, high contrast, cinematic, ultra detailed, intricate, professional",
            "negative_prompt": "painting, drawing, illustration, glitch, deformed, mutated, cross-eyed, ugly, disfigured",
        },
        {
            "name": "3D Model",
            "prompt": "professional 3d model {prompt} . octane render, highly detailed, volumetric, dramatic lighting",
            "negative_prompt": "ugly, deformed, noisy, low poly, blurry, painting",
        },
    ]
    styles = {k["name"]: (k["prompt"], k["negative_prompt"]) for k in style_list}
    STYLE_NAMES = list(styles.keys())

    CATEGORIES = ['SD15', 'SD21', 'SDXL', 'Pixart', "Other"]
    DEFAULT_CATEGORY = CATEGORIES[0]

    fire.Fire(main)



# --------------------------------------------------------
# InternVL
# Copyright (c) 2023 OpenGVLab
# Licensed under The MIT License [see LICENSE for details]
# --------------------------------------------------------
from functools import partial
from typing import Optional

import numpy as np
import torch
import torch.nn.functional as F
import torch.utils.checkpoint
from peft import LoraConfig, get_peft_model
from timm.models.layers import DropPath
from torch import nn
from transformers import GenerationConfig
from transformers.modeling_utils import PreTrainedModel
from transformers.utils import logging

from .configuration_internvl import InternVLConfig
from .modeling_intern_vit import (InternVisionEmbeddings, InternVisionEncoder,
                                  InternVisionModel)
from .modeling_qllama import LlamaForCausalLM, _expand_mask, _make_causal_mask

try:
    from .flash_attention import FlashAttention  # v1/v2
except:
    print('FlashAttention is not installed.')

logger = logging.get_logger(__name__)


class InternVLPreTrainedModel(PreTrainedModel):
    """
    An abstract class to handle weights initialization and a simple interface for downloading and loading pretrained
    models.
    """

    config_class = InternVLConfig
    base_model_prefix = 'internvl'
    supports_gradient_checkpointing = True
    _keys_to_ignore_on_load_missing = [
        r'position_ids',
    ]
    _no_split_modules = ['InternAttention', 'LlamaDecoderLayer', 'LlamaForCausalLM']
    _skip_keys_device_placement = 'past_key_values'
    _keep_in_fp32_modules = ['wo']

    def _init_weights(self, module):
        """Initialize the weights"""
        factor = self.config.initializer_range
        if isinstance(module, nn.Conv2d) or isinstance(module, nn.Embedding) or isinstance(module, nn.Linear):
            module.weight.data.normal_(mean=0.0, std=factor)
            if hasattr(module, 'bias') and module.bias is not None:
                module.bias.data.zero_()
        if isinstance(module, InternVisionEmbeddings):
            if hasattr(self.config, 'vision_config'):
                factor = self.config.vision_config.initializer_range
            nn.init.trunc_normal_(module.position_embedding, mean=0.0, std=factor)
            nn.init.trunc_normal_(module.class_embedding, mean=0.0, std=factor)
        elif isinstance(module, nn.LayerNorm):
            module.bias.data.zero_()
            module.weight.data.fill_(1.0)
        elif isinstance(module, nn.Linear) and module.bias is not None:
            module.bias.data.zero_()

    def _set_gradient_checkpointing(self, module, value=False):
        if isinstance(module, InternVisionModel):
            module.gradient_checkpointing = value
        if isinstance(module, InternVisionEncoder):
            module.gradient_checkpointing = value


class CrossAttention(nn.Module):
    def __init__(
            self, dim, num_heads=8, qkv_bias=False, qk_scale=None, attn_drop=0.,
            proj_drop=0., attn_head_dim=None, out_dim=None):
        super().__init__()
        if out_dim is None:
            out_dim = dim
        self.num_heads = num_heads
        head_dim = dim // num_heads
        if attn_head_dim is not None:
            head_dim = attn_head_dim
        all_head_dim = head_dim * self.num_heads
        self.scale = qk_scale or head_dim ** -0.5
        assert all_head_dim == dim

        self.q = nn.Linear(dim, all_head_dim, bias=False)
        self.k = nn.Linear(dim, all_head_dim, bias=False)
        self.v = nn.Linear(dim, all_head_dim, bias=False)

        if qkv_bias:
            self.q_bias = nn.Parameter(torch.zeros(all_head_dim))
            self.k_bias = nn.Parameter(torch.zeros(all_head_dim))
            self.v_bias = nn.Parameter(torch.zeros(all_head_dim))
        else:
            self.q_bias = None
            self.k_bias = None
            self.v_bias = None

        self.attn_drop = nn.Dropout(attn_drop)
        self.proj = nn.Linear(all_head_dim, out_dim)
        self.proj_drop = nn.Dropout(proj_drop)

    def forward(self, x, k=None, v=None):
        B, N, C = x.shape
        N_k = k.shape[1]
        N_v = v.shape[1]

        q_bias, k_bias, v_bias = None, None, None
        if self.q_bias is not None:
            q_bias = self.q_bias
            k_bias = self.k_bias
            v_bias = self.v_bias

        q = F.linear(input=x, weight=self.q.weight, bias=q_bias)
        q = q.reshape(B, N, 1, self.num_heads, -1).permute(2, 0, 3, 1, 4).squeeze(0)  # (B, N_head, N_q, dim)

        k = F.linear(input=k, weight=self.k.weight, bias=k_bias)
        k = k.reshape(B, N_k, 1, self.num_heads, -1).permute(2, 0, 3, 1, 4).squeeze(0)

        v = F.linear(input=v, weight=self.v.weight, bias=v_bias)
        v = v.reshape(B, N_v, 1, self.num_heads, -1).permute(2, 0, 3, 1, 4).squeeze(0)

        q = q * self.scale
        attn = (q @ k.transpose(-2, -1))  # (B, N_head, N_q, N_k)

        attn = attn.softmax(dim=-1)
        attn = self.attn_drop(attn)

        x = (attn @ v).transpose(1, 2).reshape(B, N, -1)
        x = self.proj(x)
        x = self.proj_drop(x)

        return x


class AttentiveBlock(nn.Module):

    def __init__(self, dim, num_heads, qkv_bias=False, qk_scale=None, drop=0., attn_drop=0.,
                 drop_path=0., norm_layer=nn.LayerNorm, attn_head_dim=None, out_dim=None):
        super().__init__()

        self.norm1_q = norm_layer(dim)
        self.norm1_k = norm_layer(dim)
        self.norm1_v = norm_layer(dim)
        self.cross_attn = CrossAttention(
            dim, num_heads=num_heads, qkv_bias=qkv_bias, qk_scale=qk_scale, attn_drop=attn_drop,
            proj_drop=drop, attn_head_dim=attn_head_dim, out_dim=out_dim)

        self.drop_path = DropPath(drop_path) if drop_path > 0. else nn.Identity()

    def forward(self, x_q, x_kv, pos_q, pos_k, bool_masked_pos, rel_pos_bias=None):
        x_q = self.norm1_q(x_q + pos_q)
        x_k = self.norm1_k(x_kv + pos_k)
        x_v = self.norm1_v(x_kv)
        x = self.cross_attn(x_q, k=x_k, v=x_v)

        return x


class AttentionPoolingBlock(AttentiveBlock):

    def forward(self, x):
        x_q = x.mean(1, keepdim=True)
        x_kv, pos_q, pos_k = x, 0, 0
        x = super().forward(x_q, x_kv, pos_q, pos_k, bool_masked_pos=None, rel_pos_bias=None)
        x = x.squeeze(1)
        return x


class InternVLModel(InternVLPreTrainedModel):
    config_class = InternVLConfig
    main_input_name = 'pixel_values'

    def __init__(self, config: InternVLConfig):
        super().__init__(config)

        text_hidden_size = config.qllama_config.hidden_size
        vision_hidden_size = config.vision_config.hidden_size
        clip_embed_dim = config.clip_embed_dim
        attn_pool_num_heads = config.attn_pool_num_heads
        config.qllama_config.num_query_token = config.num_query_token
        self.num_query_token = config.num_query_token
        self.label_smoothing = config.label_smoothing

        self.vision_model = InternVisionModel(config.vision_config)  # frozen
        self.qllama = LlamaForCausalLM(config.qllama_config)  # frozen
        self.query_tokens = nn.Parameter(  # trainable
            torch.zeros(1, config.num_query_token, text_hidden_size)
        )

        self.text_projection = nn.Parameter(torch.empty(text_hidden_size, clip_embed_dim))  # frozen
        self.logit_scale = nn.Parameter(torch.ones([]) * np.log(1 / 0.07))  # trainable
        self.clip_projector = AttentionPoolingBlock(  # frozen
            dim=vision_hidden_size, num_heads=attn_pool_num_heads, qkv_bias=True, qk_scale=None,
            drop=0., attn_drop=0., norm_layer=partial(nn.LayerNorm, eps=1e-5), out_dim=clip_embed_dim)
        self.clip_projector2 = AttentionPoolingBlock(  # trainable
            dim=text_hidden_size, num_heads=attn_pool_num_heads, qkv_bias=True, qk_scale=None,
            drop=0., attn_drop=0., norm_layer=partial(nn.LayerNorm, eps=1e-5), out_dim=clip_embed_dim)
        self.itm_head = nn.Linear(text_hidden_size, 2)  # trainable
        self.gradient_checkpointing = True

        # Initialize weights and apply final processing
        # self.post_init()

        if config.use_backbone_lora:
            self.wrap_backbone_lora(r=config.use_backbone_lora)
        if config.use_qllama_lora:
            self.wrap_qllama_lora(r=config.use_qllama_lora)
        if config.force_image_size:
            self.vision_model.resize_pos_embeddings(
                old_size=config.vision_config.image_size,
                new_size=config.force_image_size,
                patch_size=config.vision_config.patch_size
            )

    def wrap_backbone_lora(self, r=128, lora_alpha=256, lora_dropout=0.05):
        lora_config = LoraConfig(
            r=r,
            target_modules=['attn.qkv', 'attn.proj', 'mlp.fc1', 'mlp.fc2'],
            lora_alpha=lora_alpha,
            lora_dropout=lora_dropout,
        )
        self.vision_model = get_peft_model(self.vision_model, lora_config)
        self.vision_model.print_trainable_parameters()

    def wrap_qllama_lora(self, r=128, lora_alpha=256, lora_dropout=0.05):
        lora_config = LoraConfig(
            r=r,
            target_modules=['self_attn.q_proj', 'self_attn.k_proj', 'self_attn.v_proj', 'self_attn.o_proj',
                            'mlp.gate_proj', 'mlp.down_proj', 'mlp.up_proj'],
            lora_alpha=lora_alpha,
            lora_dropout=lora_dropout,
        )
        self.qllama = get_peft_model(self.qllama, lora_config)
        self.qllama.print_trainable_parameters()

    def get_input_embeddings(self):
        return self.qllama.get_input_embeddings()

    def set_input_embeddings(self, value):
        self.qllama.set_input_embeddings(value)

    def set_output_embeddings(self, new_embeddings):
        self.qllama.set_output_embeddings(new_embeddings)

    def get_output_embeddings(self) -> nn.Module:
        return self.qllama.get_output_embeddings()

    @torch.no_grad()
    def generate(
            self,
            pixel_values: torch.FloatTensor,
            input_ids: torch.FloatTensor,
            attention_mask: torch.LongTensor,
            generation_config: Optional[GenerationConfig] = None,
            output_hidden_states: Optional[bool] = None,
            return_dict: Optional[bool] = None,
            **generate_kwargs,
    ) -> torch.LongTensor:

        vision_outputs = self.vision_model(
            pixel_values=pixel_values,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict)
        image_embeds = vision_outputs[0]

        batch_size = image_embeds.shape[0]
        input_embeds = self.get_input_embeddings()(input_ids)
        query_tokens = self.query_tokens.repeat(batch_size, 1, 1)
        input_embeds = torch.cat([query_tokens, input_embeds], dim=1)
        image_attention_mask = torch.ones(query_tokens.size()[:-1], dtype=torch.long, device=image_embeds.device)
        attention_mask = torch.cat([image_attention_mask, attention_mask], dim=1)

        outputs = self.qllama.generate(
            inputs_embeds=input_embeds,
            attention_mask=attention_mask,
            vision_hidden_states=image_embeds,
            generation_config=generation_config,
            use_zero_attention_mask=True,
            **generate_kwargs,
        )

        return outputs

    def get_text_features(
            self,
            input_ids: torch.Tensor,
            attention_mask: torch.Tensor,
            output_attentions: Optional[bool] = None,
            output_hidden_states: Optional[bool] = None,
            return_dict: Optional[bool] = None,
    ):
        r"""
        Returns:
            text_outputs (`CausalLMOutputWithPast`, or `tuple(torch.FloatTensor)` if `return_dict=False`):
                The language model outputs. If `return_dict=True`, the output is a [`CausalLMOutputWithPast`] that
                contains the language model logits, the past key values and the hidden states if
                `output_hidden_states=True`.
        ```"""
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        input_embeds = self.get_input_embeddings()(input_ids)
        attention_mask = _expand_mask(attention_mask, input_embeds.dtype).to(
            input_embeds.device)  # [bsz, 1, tgt_seq_len, src_seq_len]
        attention_mask += _make_causal_mask(
            (attention_mask.shape[0], attention_mask.shape[2]),
            input_embeds.dtype,
            device=input_embeds.device
        )
        if type(self.qllama.model) == LlamaForCausalLM:
            outputs = self.qllama.model.model.forward_train(
                inputs_embeds=input_embeds,
                vision_hidden_states=None,
                attention_mask=attention_mask,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
            ).last_hidden_state
        else:
            outputs = self.qllama.model.forward_train(
                inputs_embeds=input_embeds,
                vision_hidden_states=None,
                attention_mask=attention_mask,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
            ).last_hidden_state
        return outputs

    def get_image_features(
            self,
            pixel_values: torch.FloatTensor,
            output_attentions: Optional[bool] = None,
            output_hidden_states: Optional[bool] = None,
            return_dict: Optional[bool] = None,
    ):
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        vision_outputs = self.vision_model(
            pixel_values=pixel_values,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict)
        image_embeds = vision_outputs[0]
        backbone_embeds = image_embeds

        batch_size = image_embeds.shape[0]
        input_embeds = self.query_tokens.repeat(batch_size, 1, 1)

        attention_mask = torch.ones(input_embeds.size()[:-1], dtype=torch.long, device=image_embeds.device)
        attention_mask = _expand_mask(attention_mask, input_embeds.dtype).to(
            input_embeds.device)  # [bsz, 1, tgt_seq_len, src_seq_len]
        if type(self.qllama.model) == LlamaForCausalLM:
            outputs = self.qllama.model.model.forward_train(
                inputs_embeds=input_embeds,
                vision_hidden_states=image_embeds,
                attention_mask=attention_mask,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
            ).last_hidden_state
        else:
            outputs = self.qllama.model.forward_train(
                inputs_embeds=input_embeds,
                vision_hidden_states=image_embeds,
                attention_mask=attention_mask,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
            ).last_hidden_state
        return backbone_embeds, outputs

    def encode_image(self, image, mode):
        if mode == 'InternVL-C':
            vision_outputs = self.vision_model(
                pixel_values=image,
                output_hidden_states=False,
                return_dict=True)
            image_embeds = vision_outputs[0]
            image_embeds = self.clip_projector(image_embeds)
        elif mode == 'InternVL-G':
            backbone_embeds, image_embeds = self.get_image_features(
                pixel_values=image,
                output_hidden_states=False,
                return_dict=True,
            )
            backbone_embeds = self.clip_projector(backbone_embeds)
            image_embeds = self.clip_projector2(image_embeds)
            # ensemble
            backbone_embeds = backbone_embeds / backbone_embeds.norm(dim=1, keepdim=True)
            image_embeds = image_embeds / image_embeds.norm(dim=1, keepdim=True)
            image_embeds = image_embeds + backbone_embeds
        else:
            raise NotImplementedError
        return image_embeds

    def encode_text(self, text):
        attention_mask = text > 0
        text_embeds = self.get_text_features(
            input_ids=text,
            attention_mask=attention_mask,
            output_attentions=False,
            output_hidden_states=False,
            return_dict=True,
        )
        text_embeds = text_embeds[torch.arange(text_embeds.shape[0]), attention_mask.sum(1) - 1]
        text_embeds = text_embeds @ self.text_projection
        return text_embeds

    def forward(self, image, text, mode='InternVL-C'):
        assert mode in ['InternVL-C', 'InternVL-G'], 'mode must be InternVL-C or InternVL-G'
        image_features = self.encode_image(image, mode)
        text_features = self.encode_text(text)

        # normalized features
        image_features = image_features / image_features.norm(dim=1, keepdim=True)
        text_features = text_features / text_features.norm(dim=1, keepdim=True)

        # cosine similarity as logits
        logit_scale = self.logit_scale.exp()
        logits_per_image = logit_scale * image_features @ text_features.t()
        logits_per_text = logits_per_image.t()

        return logits_per_image, logits_per_text


class InternVL_C(InternVLModel):

    def encode_image(self, image):
        vision_outputs = self.vision_model(
            pixel_values=image,
            output_hidden_states=False,
            return_dict=True)
        image_embeds = vision_outputs[0]
        image_embeds = self.clip_projector(image_embeds)
        return image_embeds

    def encode_text(self, text):
        attention_mask = text > 0
        text_embeds = self.get_text_features(
            input_ids=text,
            attention_mask=attention_mask,
            output_attentions=False,
            output_hidden_states=False,
            return_dict=True,
        )
        text_embeds = text_embeds[torch.arange(text_embeds.shape[0]), attention_mask.sum(1) - 1]
        text_embeds = text_embeds @ self.text_projection
        return text_embeds

    def forward(self, image, text):
        image_features = self.encode_image(image)
        text_features = self.encode_text(text)

        # normalized features
        image_features = image_features / image_features.norm(dim=1, keepdim=True)
        text_features = text_features / text_features.norm(dim=1, keepdim=True)

        # cosine similarity as logits
        logit_scale = self.logit_scale.exp()
        logits_per_image = logit_scale * image_features @ text_features.t()
        logits_per_text = logits_per_image.t()

        return logits_per_image, logits_per_text


class InternVL_G(InternVLModel):

    def encode_image(self, image):
        backbone_embeds, image_embeds = self.get_image_features(
            pixel_values=image,
            output_hidden_states=False,
            return_dict=True,
        )
        backbone_embeds = self.clip_projector(backbone_embeds)
        image_embeds = self.clip_projector2(image_embeds)
        # ensemble
        backbone_embeds = backbone_embeds / backbone_embeds.norm(dim=1, keepdim=True)
        image_embeds = image_embeds / image_embeds.norm(dim=1, keepdim=True)
        image_embeds = image_embeds + backbone_embeds
        return image_embeds

    def encode_text(self, text):
        attention_mask = text > 0
        text_embeds = self.get_text_features(
            input_ids=text,
            attention_mask=attention_mask,
            output_attentions=False,
            output_hidden_states=False,
            return_dict=True,
        )
        text_embeds = text_embeds[torch.arange(text_embeds.shape[0]), attention_mask.sum(1) - 1]
        text_embeds = text_embeds @ self.text_projection
        return text_embeds

    def forward(self, image, text):
        image_features = self.encode_image(image)
        text_features = self.encode_text(text)

        # normalized features
        image_features = image_features / image_features.norm(dim=1, keepdim=True)
        text_features = text_features / text_features.norm(dim=1, keepdim=True)

        # cosine similarity as logits
        logit_scale = self.logit_scale.exp()
        logits_per_image = logit_scale * image_features @ text_features.t()
        logits_per_text = logits_per_image.t()

        return logits_per_image, logits_per_text



from typing import Optional

import torch
from torch import nn

from .modeling_internvl import InternVLPreTrainedModel
from .modeling_qllama import LlamaForCausalLM, _expand_mask, _make_causal_mask


class InternVLTextModel(InternVLPreTrainedModel):
    def __init__(self, config):
        super().__init__(config)
        self.qllama = LlamaForCausalLM(config.qllama_config)  # frozen
        self.gradient_checkpointing = True

        text_hidden_size = config.qllama_config.hidden_size
        clip_embed_dim = config.clip_embed_dim
        self.text_projection = nn.Parameter(torch.empty(text_hidden_size, clip_embed_dim))

    def get_input_embeddings(self):
        return self.qllama.get_input_embeddings()

    def forward(
        self,
        input_ids: torch.Tensor,
        attention_mask: torch.Tensor = None,
        output_attentions=False,
        output_hidden_states=False,
        return_dict=True,
        return_pool=False,
    ):
        r"""
        Returns:
            text_outputs (`CausalLMOutputWithPast`, or `tuple(torch.FloatTensor)` if `return_dict=False`):
                The language model outputs. If `return_dict=True`, the output is a [`CausalLMOutputWithPast`] that
                contains the language model logits, the past key values and the hidden states if
                `output_hidden_states=True`.
        ```"""
        attention_mask = input_ids > 0

        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        input_embeds = self.get_input_embeddings()(input_ids)
        attention_mask = _expand_mask(attention_mask, input_embeds.dtype).to(
            input_embeds.device)  # [bsz, 1, tgt_seq_len, src_seq_len]
        attention_mask += _make_causal_mask(
            (attention_mask.shape[0], attention_mask.shape[2]),
            input_embeds.dtype,
            device=input_embeds.device
        )
        if type(self.qllama.model) == LlamaForCausalLM:
            outputs = self.qllama.model.model.forward_train(
                inputs_embeds=input_embeds,
                vision_hidden_states=None,
                attention_mask=attention_mask,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
            ).last_hidden_state
        else:
            outputs = self.qllama.model.forward_train(
                inputs_embeds=input_embeds,
                vision_hidden_states=None,
                attention_mask=attention_mask,
                output_attentions=output_attentions,
                output_hidden_states=output_hidden_states,
                return_dict=return_dict,
            ).last_hidden_state

        if return_pool:
            attention_mask = input_ids > 0
            text_embeds = outputs[torch.arange(outputs.shape[0]), attention_mask.sum(1) - 1]
            text_embeds = text_embeds @ self.text_projection
            return [outputs, text_embeds]

        return [outputs]
